from os.path import join
import sys, traceback
import os
import random
import time
try:
    import configparser
except ImportError:
    # Python 2.x fallback
    import ConfigParser as configparser

# this import trigger a connection to Hecuba database
# from hecuba_lib import StorageObj

# COMPSs/PyCOMPSs imports
from pycompss.api.task import task
from pycompss.api.binary import binary
from pycompss.api.constraint import constraint
from pycompss.api.mpi import mpi
from pycompss.api.api import compss_wait_on
from pycompss.api.api import compss_barrier
from pycompss.api.api import TaskGroup
from pycompss.api.api import compss_barrier_group
from pycompss.api.parameter import *
from pycompss.api.on_failure import on_failure
from pycompss.api.exceptions import COMPSsException

# project imports
from esm_ensemble_init import *
from hecuba_lib.esm_dynamic_analysis_results import *


# things we need:
# 1 - every 30 seconds, check the state of state of the members and determine which members to prune
# 2 - every n time steps,  perform the analysis of the model data

@on_failure(management='IGNORE', returns=0)
@task( returns=int, exp_id={Type:IN})
def esm_dynamic_analysis(exp_id):

  try:

    print("######################## performing dynamic analysis for experiment " + exp_id + "###################")
    # create a dummy object
    # TODO: here is the launching point of the analysis, it will be a PyCOMPSs task
    ds = esm_dynamic_analysis_results()
    ds["1968"] = False
    ds.make_persistent(exp_id  + ".esm_dynamic_analysis")

  except COMPSsException:
      pass

# @on_failure(management='IGNORE', returns=0)
# @task( returns=int, exp_id={Type:IN}, log_pruning={Type: FILE_OUT, StdIOStream: STDOUT})
# def esm_pruning(exp_id, log_pruning):
#    try:
#      print("######################## launching pruning " + str(time.time()) + "########################")
#      # 1 - run the analysis workflow
#      esm_dynamic_analysis(exp_id)
#      # 2 - check the results
#      get_members_state(exp_id)
#      return 0
#    except Exception:
#      print("Exception in esm_pruning:")
#      print("-" * 60)
#      traceback.print_exc(file=sys.stdout)
#      print("-" * 60)

# for testing purposes
if __name__ == "__main__":
    print("Running Dynamic analysis Agent")
    # first parameter: number of ensemble members
    exp_id = str(sys.argv[1])
    exp_settings = compss_wait_on(esm_ensemble_init(exp_id, False))
    top_working_dir = exp_settings['common']['top_working_dir']
    exp_flag = top_working_dir + "/" + exp_id + ".finished"
    log_file = exp_id + "_pruning_log.out"

    while (not os.path.exists(exp_flag)):
        compss_wait_on(esm_dynamic_analysis(exp_id, log_file))
        time.sleep(5)

    file = open(log_file, "w")
    file.write("Shutting down Pruning analysis agent")
    file.close()

