from os.path import join
import sys
import os
import random
import shutil

try:
    import configparser
except ImportError:
    # Python 2.x fallback
    import ConfigParser as configparser
from time import time

# COMPSs/PyCOMPSs imports
from pycompss.api.task import task
from pycompss.api.binary import binary
from pycompss.api.constraint import constraint
from pycompss.api.mpi import mpi
from pycompss.api.api import compss_wait_on
from pycompss.api.api import compss_barrier
from pycompss.api.api import TaskGroup
from pycompss.api.api import compss_barrier_group
from pycompss.api.parameter import *
from pycompss.api.on_failure import on_failure
from pycompss.api.exceptions import COMPSsException

# project imports
from esm_ensemble_init import *
from hecuba import StorageObj

# Hecuba datamodel imports
from hecuba_lib.esm_dynamic_analysis_results import *
#from hecuba_lib import fesom_datamodel

@on_failure(management='CANCEL_SUCCESSORS')
@mpi(binary="${FESOM_EXE}", runner="mpirun", processes="${FESOM_CORES}", working_dir="{{working_dir_exe}}", fail_by_exit_value=True)
@task(log_file={Type:FILE_OUT, StdIOStream:STDOUT}, working_dir_exe={Type:INOUT, Prefix:"#"})
def esm_simulation(log_file, working_dir_exe):
   pass

def esm_member_checkpoint(exp_id, sdate, checkpoint_log):
   #retrieve from Hecuba the last status of the ensemble members produced by the analysis (running in parallel)
   ensemble_status = esm_dynamic_analysis_results.get_by_alias(exp_id + "." + sdate)
   return ensemble_status




#TODO: add timeout to the SIMS so these execute the intended time segments (chunks)
def esm_member(exp_id, sdate, n_sims, top_working_dir):

   try:
      for sim in range( 1, n_sims+1 ):
         working_dir_exe = top_working_dir + "/" + sdate
         log = working_dir_exe + "/" + "fesom2_" + str(exp_id) + "_" + str(sdate) + ".out"
         print( "################## launching simulation " + sdate + "." + str(sim) + "in " + working_dir_exe + "######################")
         compss_wait_on(esm_simulation(log, working_dir_exe))
         # check the state of the member, for the first one there is nothing to check
         to_continue = esm_member_checkpoint(exp_id, sdate, log)
         # if it should be aborted we throug an exception
         if not to_continue:
            raise COMPSsException("Member diverged - Aborting member " + str(sdate) )

   except COMPSsException:
       # React to the exception (maybe calling other tasks or with other parameters)
       #TODO: add logic to remove all data from working dirs/output folders
      esm_member_disposal(exp_id, sdate, top_working_dir)
      pass


def esm_member_disposal(exp_id, sdate, top_working_dir):

   # TODO: remove hecuba data aswell of the concerned aborted member
   # path to be removed
   path = os.path.join("/home/bsc32/bsc32044/results/output_core2/", sdate)
   # removing directory
   shutil.rmtree(path)




# function to run ESM ensemble, each one should have different start dates or initial conditions/ forcings
# for making sense
def esm_ensemble(exp_id):

   # 1 - Creation of group
   with TaskGroup(str(exp_id), False):
      #3 - Launch the ensemble members
      top_working_dir = exp_settings['common']['top_working_dir']
      sdates_list = (exp_settings['common']['ensemble_start_dates']).split()
      for sdate in sdates_list:
         # working_dir_exe = top_working_dir + "/" + sdate
         # log = working_dir_exe + "/" + "fesom2_" + str(exp_id) + "_" + str(sdate) + ".out"
         # print("################################################### launching simulation in " + working_dir_exe + "###################################################")
         # esm_simulation(log, working_dir_exe)
         # todo: add parameter to load the number of simulation chunks to run (based on frecuency)
         n_sims = 4
         esm_member(exp_id, sdate, n_sims, top_working_dir)

   compss_barrier_group(str(exp_id))
   # create flag file to stop the parallel process
   run_finished_flag =  str(exp_id) + ".finished"
   open(top_working_dir + "/" + run_finished_flag, 'a').close()


if __name__ == "__main__":
   print("Running FESOM2 -Pycompss")
   # 0 - create a ramdon nr for the experiment id
   #exp_id = random.randint(100000, 999999)
   exp_id = str(sys.argv[1])
   # prepare folder structure/configuration
   # todo: check if the barrier can be removed
   exp_settings = compss_wait_on(esm_ensemble_init(exp_id, True))
   # exp_settings =  compss_wait_on(exp_settings)
   print("##################################### Initialization completed ####################################")
   # run the experiment
   esm_ensemble(exp_id)
